@@ -20,9 +20,10 @@ class MLEnv():
 
     :param platform: Known platforms are: `'tf'` (tensorflow), `'pt'` (pytorch), and `'jax'`
     :param accelerator: known accelerators are: `'fastest'` (pick best available hardware), `'cpu'`, `'gpu'`, `'tpu'`.
+    :param old_disable_eager: default 'False', on True, old v1 compatibility layer is used to disable eager mode. According to rumors that might in resulting old codepaths being used?
     """
 
-    def __init__(self, platform='tf', accelerator='fastest'):
+    def __init__(self, platform='tf', accelerator='fastest', old_disable_eager=False):
         self.log = logging.getLogger('MLEnv')
         self.known_platforms = ['tf', 'pt', 'jax']
         self.known_accelerators = ['cpu', 'gpu', 'tpu', 'fastest']
@@ -92,30 +93,42 @@ class MLEnv():
                             self.log.info("You got old TPU v2 which is limited to 8GB Ram.")
                     self.tpu_type = tpu_type
                     self.log.debug("TPU strategy available")
-                    tf.compat.v1.disable_eager_execution()
-                    self.log.debug("TPU: eager execution disabled!")
+                    if old_disable_eager is True:
+                        self.log.debug("Switching to non-eager mode")
+                        tf.compat.v1.disable_eager_execution()
+                        self.log.debug("TPU: eager execution disabled using old compat.v1 API!")
             if self.is_tpu is False:
                 if accelerator == 'gpu' or accelerator == 'fastest':
                     try:
-                        tf.config.experimental.list_physical_devices('GPU')
-                        self.is_gpu = True
+                        dev_list=tf.config.experimental.list_physical_devices('GPU')
+                        if dev_list is not None and len(dev_list)>0:
+                            self.is_gpu = True
+                        else:
+                            self.log.debug("GPU not available")
+                            self.is_gpu = False
                     except RuntimeError as e:
                         self.log.debug(f"GPU not available: {e}")
                         self.is_gpu = False
                     if self.is_gpu is True:
                         try:
-                            self.gpu_type=tf.config.experimental.get_device_details(tf.config.list_physical_devices('GPU')[0])['device_name']
+                            dev_list=tf.config.list_physical_devices('GPU')
+                            if dev_list is not None and len(dev_list)>0:
+                                self.gpu_type=tf.config.experimental.get_device_details(dev_list[0])['device_name']
+                            else:
+                                self.is_gpu=False
                         except Exception as e:
                             self.log.warning(f"Could not get GPU type: {e}")
-                        try: 
-                            card = subprocess.run(['nvidia-smi'], stdout=subprocess.PIPE).stdout.decode('utf-8').split('\n')
-                            if len(card)>=8:
-                                self.gpu_memory=card[9][33:54].strip()
-                            else:
-                                self.log.warning(f"Could not get GPU type, unexpected output from nvidia-smi, lines={len(card)}, content={card}")
-                        except Exception as e:
-                            self.log.debug(f"Failed to determine GPU memory {e}")
-                        self.log.debug("GPU available")
+                            self.is_gpu=False
+                        if self.is_gpu is True:
+                            try: 
+                                card = subprocess.run(['nvidia-smi'], stdout=subprocess.PIPE).stdout.decode('utf-8').split('\n')
+                                if len(card)>=8:
+                                    self.gpu_memory=card[9][33:54].strip()
+                                else:
+                                    self.log.warning(f"Could not get GPU type, unexpected output from nvidia-smi, lines={len(card)}, content={card}")
+                            except Exception as e:
+                                self.log.debug(f"Failed to determine GPU memory {e}")
+                            self.log.debug("GPU available")
             if self.is_gpu is False: 
                 self.log.info("No GPU or TPU available, this is going to be very slow!")
         if platform == 'jax':
@@ -208,6 +221,17 @@ class MLEnv():
                             if accelerator != 'fastest':
                                 return
                 if accelerator == 'gpu' or accelerator == 'fastest':
+                    if 'darwin' in sys.platform:
+                        try:
+                            if torch.has_mps:
+                                self.is_gpu = True
+                                self.log.debug("Pytorch MPS acceleration detected.")
+                                self.gpu_type="MPS Metal accelerator"
+                                self.gpu_memory="system memory"
+                                self.log.debug(f"Pytorch MPS acceleration detected: MPS={torch.has_mps}")
+                                return
+                        except:
+                            pass
                     try:
                         import torch.cuda
                         if torch.cuda.is_available():
